import json
import logging
from abc import ABC, abstractmethod
from enodo.jobs import JOB_TYPE_BASE_SERIES_ANALYSIS, JOB_TYPE_FORECAST_SERIES, JOB_TYPE_DETECT_ANOMALIES_FOR_SERIES

class EnodoJobDataModel():

    def __init__(self, **kwargs):
        self._dict_values = kwargs
        if not self.validate():
            raise Exception("invalid data for packaga data")
        
        # self.__dict__ = json.loads(self._raw_data)

    def validate(self):
        if self.required_fields is not None:
            for key in self.required_fields:
                if key not in self._dict_values.keys():
                    logging.info(f"Missing '{key}' in enodo job data model data")
                    return False
        return "model_type" in self._dict_values.keys()

    @property
    @abstractmethod
    def required_fields(self):
        """ return the sound the animal makes """

    def get(self, key):
        return self._dict_values.get(key)

    def serialize(self):
        return json.dumps(self._dict_values)

    @classmethod
    def unserialize(cls, json_data):
        data = json.loads(json_data)
        model_type = data.get("model_type")

        if model_type == "forecast_response":
            return EnodoForecastJobResponseDataModel(**data)
        elif model_type == "anomaly_response":
            return EnodoDetectAnomaliesJobResponseDataModel(**data)
        elif model_type == "base_response":
            return EnodoBaseAnalysisJobResponseDataModel(**data)
        elif model_type == "forecast_response":
            return EnodoForecastJobResponseDataModel(**data)
        elif model_type == "job_request":
            return EnodoJobRequestDataModel(**data)
        

        return None


class EnodoJobRequestDataModel(EnodoJobDataModel):

    def __init__(self, **kwargs):
        kwargs['model_type'] = "job_request"
        super().__init__(**kwargs)

    @property
    def required_fields(self):
        return [
            "job_id",
            "job_type",
            "series_name",
            "series_config",
            "global_series_config"
        ]


class EnodoForecastJobResponseDataModel(EnodoJobDataModel):

    def __init__(self, **kwargs):
        kwargs['model_type'] = "forecast_response"
        super().__init__(**kwargs)

    @property
    def required_fields(self):
        return [
            "successful",
            "forecast_points"
        ]

class EnodoDetectAnomaliesJobResponseDataModel(EnodoJobDataModel):

    def __init__(self, **kwargs):
        kwargs['model_type'] = "anomaly_response"
        super().__init__(**kwargs)

    @property
    def required_fields(self):
        return [
            "successful",
            "flagged_anomaly_points"
        ]

class EnodoBaseAnalysisJobResponseDataModel(EnodoJobDataModel):

    def __init__(self, **kwargs):
        kwargs['model_type'] = "base_response"
        super().__init__(**kwargs)

    @property
    def required_fields(self):
        return [
            "successful",
            "trend_slope_value",
            "noise_value",
            "has_seasonality",
            "health_of_series"
        ]


class EnodoModel:
    __slots__ = ('model_name', 'model_arguments')

    def __init__(self, name, model_arguments):
        """
        :param name:
        :param model_arguments:  in form of  {'key': True} Where key is argument name and
                                    value is wether or not it is mandatory
        """
        self.model_name = name
        self.model_arguments = model_arguments

    @classmethod
    def to_dict(cls, model):
        return {
            'model_name': model.model_name,
            'model_arguments': model.model_arguments
        }

    @classmethod
    def from_dict(cls, model):
        return EnodoModel(model.get('model_name'),
                          model.get('model_arguments'))





import json

from . import ConfigModel
from enodo.jobs import JOB_TYPES


class SeriesConfigModel(ConfigModel):

    __slots__ = ('job_models', 'job_schedule', 'min_data_points', 'model_params')
    
    def __init__(self, job_models, job_schedule, min_data_points=None, model_params=None):
        """
        Create new Series Config
        :param job_models: dict of job(key) and model name(value)
        :param job_schedule: dict of job(key) and n_new_points(value)
        :param model_params: dict of model(key) and dict(value)
        :return:
        """

        if not isinstance(job_models, dict):
            raise Exception("Invalid series config")

        for job in job_models:
            if job not in JOB_TYPES:
                raise Exception("Invalid series config")

        if not isinstance(job_schedule, dict):
            raise Exception("Invalid series config")

        self.job_models = job_models
        self.job_schedule = job_schedule
        self.min_data_points = min_data_points
        self.model_params = model_params

    def get_model_for_job(self, job_type):
        if job_type not in self.job_models:
            return False
        
        return self.job_models.get(job_type)

    @classmethod
    def from_dict(cls, data):
        return cls(**data)

    def to_dict(self):
        return {
            'job_models': self.job_models,
            'job_schedule': self.job_schedule,
            'min_data_points': self.min_data_points,
            'model_params': self.model_params
        }